/**************************************************************
*
*   Copyright 2004 NetBurner, Inc.  
*   ALL RIGHTS RESERVED
*
*   Permission is hereby granted to purchasers of NetBurner 
*   Hardware to use or modify this computer program  
*   for any use as long as the resultant program is only  
*   executed on NetBurner provided hardware.
*   
*   No other rights to use this program or its derivatives
*   in part or in whole are granted.
*
*   It may be possible to license this or other NetBurner 
*   software for use on non NetBurner Hardware. Please contact
*   Licensing@NetBurner.com for more information. 
*   
*   NetBurner makes no representation or warranties with respect
*   to the performance of this computer program, and specifically 
*   disclaims any responsibility for any damages, special or 
*   consequential, connected with the use of this program.
*   
*-----------------------------------------------------------------
*
*   NetBurner, Inc.
*   5405 Morehouse Drive, Suite 350
*   San Diego CA, 92121
*
*   Information is available at our main Web Site:
*   http://www.NetBurner.com
*   or via E-Mail at: Info@NetBurner.com
*
*   Support and Registration Web Site: http://Support.NetBurner.com 
*   Support is available (after Registration) via E-Mail at:  
*   Support@NetBurner.com
* 
******************************************************************/



/*-------------------------------------------------------------------------
 The following program will demonstrate how to use an external interrupt
 on the Coldfire 5272 processor for the NetBurner Mod5272 platform.
 Each time the IRQ button is pressed on the Mod5272 Carrier board
 an interrupt will occur, and the number of interrupts will be shown
 on the Carrier board's LED display. You can also use MTTTY to view 
 the number of times that the IRQ button is pressed.
 -------------------------------------------------------------------------*/

#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <cfinter.h>
#include <../Mod5272/System/sim5272.h>

extern "C" { 
    void UserMain(void * pd);
    void putdisp(unsigned short w);
}

/* Helper function to write a decimal number out to the LED display */
void putdecimaldisp(WORD w)
{
    WORD wo=(w/1000)<<12;
    wo+=((w/100) %10)<<8;
    wo+=((w/10) %10)<<4;
    wo+=(w %10);
    putdisp(wo);
}

/* Variables the interrupt routine will touch */
volatile WORD isrcount;
OS_SEM ButtonSem;

/* The interrupt routine */
INTERRUPT(Button_hit_Isr, 0x2400)
{
    sim.icr1 |= 0x80000000;	// clear IRQ 1
    putleds(isrcount);
    putdecimaldisp(isrcount++);
    OSSemPost(&ButtonSem);
}

/* This feature adds application identification for IP Setup Tool 
and AutoUpdate Find */
const char * AppName="Mod5272 IRQ Example";

/* UserMain */
void UserMain(void * pd)
{
 InitializeStack();
 if (EthernetIP==0)GetDHCPAddress(); 
 OSChangePrio(MAIN_PRIO);
 EnableAutoUpdate();
 OSSemInit(&ButtonSem,0);
 isrcount=0;
  
 /* Setup IRQ 1 */

 /* Set the IRQ edge to trigger on, see 5272 UM section 7.2.4 
 Beware, early 5272 UMs had the sense of this edge wrong */
 sim.pitr=0; 
 
 /* IRQ 1 enable for IRQ level 4, see 5272 UM section 7.2.2.1 */
 sim.icr1 |= 0xC0000000;	
 
 /* Put our ISR function in the vector table */
 /* The number 65 came from table 7-8 in the 5272 UM */ 
 vector_base.table[65]=(long)&Button_hit_Isr;  

 putdecimaldisp(0);
 putleds(0);

 iprintf("Application started\n");
 while (1)
  {
   OSSemPend(&ButtonSem,0);
   iprintf("The button was hit, (%d times)\r\n",isrcount);
 }
}



